#!/usr/bin/python3

import os,sys
from ReadSagittalPD import ReadSagittalPDs
from nn import classificationGrade
from nn import superResolution
from nn import segmentationJoint
from nn import segmentationSwollen
from nn import inpainting

import SimpleITK as sitk
import numpy as np

class mainLogic(object):
    def __init__(self):
        super(mainLogic, self).__init__()
        self.gradeNN = classificationGrade()
        self.superResolutionNN = superResolution()
        self.segmentationJointNN = segmentationJoint()
        self.segmentationSwollenNN = segmentationSwollen()
        self.inpaintingNN = inpainting()
        self.__image = None
        self.__seg = None
        self.__grade = None
        self.__inpainted = None
        self.__swollen = None
        self.__superResolution = None
        self.__reference = None
    
    def getImage(self):
        return self.__image

    def setImage(self, image):
        if image is None:
            return False
        self.__image = image
        self.__seg = None
        self.__grade = None
        self.__inpainted = None
        self.__swollen = None
        self.__superResolution = None
        self.__reference = None
        return True

    def getSeg(self):
        assert self.__image is not None, 'No image loaded'
        if self.__seg is not None:
            return self.__seg
        seg = self.segmentationJointNN(self.__image)
        self.__seg = seg
        return self.__seg

    def getGrade(self):
        assert self.__image is not None, 'No image loaded'
        if self.__grade is not None:
            return self.__grade
        grade = self.gradeNN(self.__image)
        self.__grade = grade
        return grade

    def getInpainted(self):
        assert self.__image is not None, 'No image loaded'
        assert self.__swollen is not None, 'Predict swollen area first'
        if self.__inpainted is not None:
            return self.__inpainted
        inpainted = self.inpaintingNN(self.__image, self.__swollen)
        self.__inpainted = inpainted
        return self.__inpainted

    def getSwollen(self):
        threshold = 0.5
        assert self.__image is not None, 'No image loaded'
        assert self.__grade is not None, 'Predict grade first'
        if self.__swollen is not None:
            return self.__swollen
        swollen = self.segmentationSwollenNN(self.__image)
        positive = self.__grade[:,1] >= threshold
        array = sitk.GetArrayFromImage(swollen)
        # negative => no swollen area
        for i in range(positive.shape[0]):
            if positive[i] == False:
                array[i,:,:] = np.zeros(array.shape[1::])
        swollen = sitk.GetImageFromArray(array)
        swollen.CopyInformation(self.__image)
        self.__swollen = swollen
        return self.__swollen

    def getSuperResolution(self):
        assert self.__image is not None, 'No image loaded'
        if self.__superResolution is not None:
            return self.__superResolution
        superResolution = self.superResolutionNN(self.__image)
        self.__superResolution = superResolution
        return self.__superResolution

    def getReference(self):
        assert self.__image is not None, 'No image loaded'
        assert self.__swollen is not None, 'Swollen area not segmented'
        assert self.__inpainted is not None, 'Inpainted image not generated'
        assert self.__superResolution is not None, 'SR image not generated'
        if self.__reference is not None:
            return self.__reference
        # combine SR and Inpainting
        srArray = sitk.GetArrayFromImage(self.__superResolution)
        inpaintedArray = sitk.GetArrayFromImage(self.__inpainted)
        swollenArray = sitk.GetArrayFromImage(self.__swollen)
        for i in range(swollenArray.shape[0]):
            inpaintedArray[i] = srArray[i] * (swollenArray[i] == 0) + \
                    inpaintedArray[i] * (swollenArray[i] != 0)
        reference = sitk.GetImageFromArray(inpaintedArray)
        reference.CopyInformation(self.__inpainted)
        self.__reference = reference
        return self.__reference

if __name__ == '__main__':
    import time
    import torch
    images = ReadSagittalPDs('/home/kylexuan/data/data/cartilage_origin/CartilageData_FenJin/SixthPeopleHospitalPD_OriDicom/A103327625')
    logic = mainLogic()
    logic.setImage(images[0])
    logic.getGrade()
    logic.getSeg()
    logic.getSwollen()
    logic.getSuperResolution()
    logic.getInpainted()
    logic.getReference()
    #print('sleeping')
    #time.sleep(10)
    #print('freeing cache')
    #torch.cuda.empty_cache()
    #time.sleep(10)
    #logic.loadImage('/home/kylexuan/data/data/cartilage_origin/CartilageData_FenJin/SixthPeopleHospitalPD_OriDicom/A103327625')
    #logic.getGrade()
    #logic.getSeg()
    #logic.getSwollen()
    #logic.getSuperResolution()
    #logic.getInpainted()
    #logic.getReference()
